#include <iostream>

#include <SDL.h>
#include <SDL_thread.h>

#include "../swarm/swarm.h"
#include "../pipes/streamPipe.h"
#include "streamElement.h"
#include "swarmElement.h"

SwarmElement::SwarmElement(
	StreamPipe* out,
	double linesPerSecond, unsigned int _lines, double _gain, bool _doAbs,
	unsigned int _Dims, unsigned int GnatCount, double accelFactor
    ) : StreamElement( 0, out ), Dims( _Dims ),
	    lineTime( 1.0 / linesPerSecond ), lines( _lines ),
	    gain( _gain / (double)( GnatCount + 1 ) ), doAbs( _doAbs )
{
    this->swarm = new Swarm( this->Dims, GnatCount, accelFactor );
    this->coefs = new double[ this->Dims ];
    this->outBuffer = new unsigned char[ this->Dims ];
}

SwarmElement::~SwarmElement( void )
{
    delete[] this->outBuffer;
    delete[] this->coefs;
    delete this->swarm;
}

void
SwarmElement::clearCoefs( void )
{
    for ( unsigned int ii=0; ii < this->Dims; ++ii ) {
	this->coefs[ ii ] = 0.0;
    }
}

void
SwarmElement::incCoefs( const double* incs )
{
    for ( unsigned int ii=0; ii < this->Dims; ++ii ) {
	this->coefs[ ii ] += incs[ ii ];
    }
}

void
SwarmElement::emitCoefs( void )
{
    for ( unsigned int ii=0; ii < this->Dims; ++ii ) {
	int val = (int)( this->gain * this->coefs[ ii ] );

	if ( this->doAbs ) {
	    if ( val < 0 ) {
		val = -val;
	    }
	} else if ( val < 0 ) {
	    val = 0;
	}

	if ( val > 255 ) {
	    val = 255;
	}

	this->outBuffer[ ii ] = (unsigned char)val;
    }

    this->out->write( this->outBuffer, this->Dims );
}

int
SwarmElement::run( void )
{
    unsigned int GnatCount = this->swarm->getGnatCount();

    this->out->write( &this->Dims, sizeof( this->Dims ) );

    for ( unsigned int ii=0; ( this->lines == 0 ) or ii < this->lines; ++ii ) {
	this->clearCoefs();

	this->incCoefs( this->swarm->getWaspPosition() );

	for ( unsigned int jj=0; jj < GnatCount; ++jj ) {
	    this->incCoefs( this->swarm->getGnatPosition( jj ) );
	}

	this->emitCoefs();
	this->swarm->update( lineTime );
    }

    return 0;
}
